/*******************************************************************************

  Copyright (c) 2001-2005, Intel Corporation
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
   1. Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
  
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
  
   3. Neither the name of the Intel Corporation nor the names of its
      contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/* $OpenBSD: ixgb_ids.h,v 1.3 2008/02/19 18:47:18 brad Exp $ */

#ifndef _IXGB_IDS_H_
#define _IXGB_IDS_H_

/**********************************************************************
** The Device and Vendor IDs for 10 Gigabit MACs
**********************************************************************/

#define INTEL_VENDOR_ID         0x8086
#define INTEL_SUBVENDOR_ID      0x8086
#define SUN_VENDOR_ID           0x108E
#define SUN_SUBVENDOR_ID        0x108E

#define IXGB_DEVICE_ID_82597EX      0x1048
#define IXGB_DEVICE_ID_82597EX_SR   0x1A48
#define IXGB_DEVICE_ID_82597EX_LR   0x1B48
#define IXGB_SUBDEVICE_ID_A11F  0xA11F
#define IXGB_SUBDEVICE_ID_A01F  0xA01F

#define IXGB_DEVICE_ID_82597EX_CX4   0x109E
#define IXGB_SUBDEVICE_ID_A00C  0xA00C
#define IXGB_SUBDEVICE_ID_A01C  0xA01C   
#define IXGB_SUBDEVICE_ID_7036  0x7036

#endif /* #ifndef _IXGB_IDS_H_ */
/* End of File */
