/*	$OpenBSD: if_em_soc.h,v 1.2 2015/06/04 18:33:41 dms Exp $	*/

/*
 * Copyright (c) 2009 Dariusz Swiderski <sfires@sfires.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


void *em_lookup_gcu(struct device *);
int em_attach_miibus(struct device *self);

int gcu_miibus_readreg(struct em_hw *, int, int);
void gcu_miibus_writereg(struct em_hw *, int, int, int);
void gcu_miibus_statchg(struct device *);

#define GCU_MAX_ATTEMPTS	64
#define GCU_CMD_DELAY		50 /* microseconds */

#define RTL8211_E_PHY_ID  0x001CC912

